;***************************************************************************
;* 							L E D - F A D E R 
;*************************************************************************** 
;*
;* Autor:		Mario Richter
;* Dateiname:	"LED-Fader.asm"
;* Titel:		LED-Fader Applikation
;* Datum:       August 2005
;* Target-MCU:	ATtiny26
;*
;*
;* BESCHREIBUNG
;*
;* Das Programm erzeugt einen Farbwechsel mit Hilfe 8 verschiedenfarbiger 
;* LEDs. Die Dauer des bergangs (1..10s) nd der Pause (10..120s) ist mit
;* zwei Potis einstellbar, ein Start/Stop-Taster toggelt den Betrieb des 
;* Wechslers. 
;*
;* ANSCHLUSSBELEGUNG
;*
;*		LEDs 1..8					PA0..PA7
;*		Poti bergangsdauer			PB6 (ADC9, 0..5V)
;*		Poti Pausenzeit				PB5	(ADC8, 0..5V)
;*		Taster Start/Stop			PB4 (LOW active)
;*
;*		ISP-Schnittstelle			PB0 (MOSI), PB1 (MISO), 
;*									PB2 (SCK), PB7 (/RESET), GND
;*
;***************************************************************************


.include "tn26def.inc"


.def temp 		= r16							; Universal-Register
.def status 	= r17							; Status-Register 
.def active_led	= r18							; Bitposition der aktuellen LED (1..128)
.def count_l	= r19							; 16 Bit-Counter
.def count_h	= r20			
.def taster		= r21							; Entprellzhler 
.def fade_int	= r22							; Fading-Intervall [*500ms]
.def adc_int	= r23							; ADC-Intervallzhler
.def pause		= r24							; Pausenzhler

.equ FADE		= 0								; Status-Bit 0: FADE
.equ START 		= 1								; Status-Bit 1: START
.equ INT_FLAG	= 2								; Status-Bit 2: Interrupt-Flag

.equ TAST_THRES	= 5								; Taster-Entprellzeit [*2ms]
.equ ADC_INTVL	= 250							; ADC-Intervall [*2ms]
.equ PAUSE_MIN	= 0x13							; min. Pausenzeit [*0,512s]
.equ PAUSE_MAX	= 0xEA							; max. Pausenzeit [*0,512s]


.org 0x00
		rjmp 	main							; Hauptprogramm

.org OC1Aaddr
		rjmp	t_comp							; Output Compare

.org OVF1addr
		rjmp	t_ovf							; Timer-Overflow




; Hauptroutine
; ************

main:
		ldi 	temp, RAMEND					; Stackpointer initialisieren
		out 	SP, temp
		ldi 	temp, 0xFF						; PortA alles Ausgnge
		out 	DDRA, temp

		; AD-Wandler konfigurieren

		ldi		temp, (1<<ADLAR)				; ADC-Ergebnis links ausrichten
		ori		temp, (1<<MUX3)|(1<<MUX0)		; ADC-Kanal 9 (Fading-Poti) auswhlen
		out		ADMUX, temp
		ldi 	temp, (1<<ADEN)					; AD-Wandler aktivieren
		ori		temp, (1<<ADPS1)|(1<<ADPS0)		; Prescaler auf 8 (1MHz/8 = 125kHz)
		out		ADCSR, temp		
		
		; Timer konfigurieren		

		ldi 	temp, (1<<CTC1)					; Clear Timer on Compare
		ori		temp, (1<<CS12)					; 8-fach Prescaler (1MHz/8 = 125kHz)
		out		TCCR1B, temp					; 
		ldi 	temp, 250						; 250 in Compare-Register 
		out		OCR1C, temp						; (250/125kHz = 2ms bzw. 500Hz)
		ldi 	temp, (1<<OCIE1A)|(1<<TOIE1)	; OC1A- und TOIE1-Interruptmaske
		out		TIMSK, temp

		; Startwerte fr Register

		cbr		status, (1<<FADE)				; Am Anfang LED1 an (kein berblenden)
		sbr		status, (1<<START)				; Start-Bit setzen
		cbr		status, (1<<INT_FLAG)			; Interrupt-Flag zurcksetzen
		ldi		active_led, 1					; Beginn mit erster LED
		out		PORTA, active_led				; LED einschalten

		clr		count_l							; Zhler lschen
		clr		count_h
		clr		taster

		rcall 	adcread							; Erste Schwellwerte vom ADC holen
	
		sei										; Interrupts aktivieren

		; Ablaufsteuerung 

m_loop:	sbrs	status, INT_FLAG				; INT_FLAG prfen
		rjmp	m_done							; 	wenn nicht gesetzt, weiter zu m_done

		; Folgender Abschnitt wird alle 2ms ausgefhrt
		; Bei Freigabe (START = 1) Counter weiterzhlen

		cbr 	status, (1<<INT_FLAG)			; INT_FLAG zurcksetzen

		sbrs	status, START					; Start freigegeben ?
		rjmp	tast							;	nein -> weiter zu tast

		cpi 	count_l, 255					; count_l berprfen
		brlo	mk_1							;	< 255: kein berlauf
		clr 	count_l							;	berlauf: count_l lschen, count_h++
		inc		count_h
		rjmp 	mk_2
mk_1:	inc		count_l							; kein berlauf: count_l erhhen

		; Auf Erreichen der Schwellen berprfen

mk_2:	sbrs	status, FADE					; Modus-berprfung
		rjmp	nofade

		cp		count_l, fade_int				; count_l >= fade_int ?
		brlo	tast							; 	wenn nicht, weiter zu tast
		clr		count_l							;	ja, dann count_l lschen
		in		temp, OCR1A
		inc 	temp							; OCR1A-Wert erhhen
		cpi 	temp, 255						; Endwert erreicht ?
		breq	mk_3							; 	ja, dann weiter zu mk_3
		out		OCR1A, temp						;	nein, dann neuen Wert schreiben
		rjmp 	tast

mk_3:	clr		count_l							; Schwelle erreicht: Zhler lschen
		cbr		status, (1<<FADE)				; Fade-Modus verlassen
		ldi		temp, 0							; OCR1A auf null zurcksetzen
		out 	OCR1A, temp
		rol		active_led						; Auf nchste LED weiterschalten
		brcc	led_an							; war letzte LED erreicht ?
		ldi 	active_led, 1					; 	dann erste anschalten
led_an:	out		PORTA, active_led				; LED anschalten
		rjmp	tast

nofade: cp		count_h, pause					; count_h >= pause ?
		brlo	tast							;	wenn nicht, weiter zu tast

		clr		count_l							; Schwelle erreicht: Zhler lschen
		clr 	count_h
		sbr		status, (1<<FADE)				; Pausen-Modus verlassen

		; Taster-Abfrage und Entprellroutine

tast:	sbic	PINB, PORTB4					; Taster abfragen
		rjmp	notast							;	nicht gedrckt -> weiter zu notast
		cpi		taster, TAST_THRES				; 	gedrckt: Zhler schon an Schwelle ?
		breq	adconv							;		ja, dann fertig
		inc		taster							; 		ansonsten weiterzhlen
		cpi		taster, TAST_THRES				; Zhler jetzt an Schwelle ?
		brne	adconv							;	nein, dann fertig

		; START-Flag toggeln; falls nach einer Pause neu gestartet wird, Zhler
		; vorstellen, damit gleich mit der nchsten berblendung begonnen wird

		sbrc	status, START					; START-Flag gelscht ?
		rjmp	mk_4							;	nein, dann zu mk_4
		sbr		status, (1<<START)				;	ja, dann erst mal START-Flag setzen
		sbrc	status, FADE					; Ist gerade eine Pause ?
		rjmp 	adconv							; 	nein, dann fertig
		mov 	temp, pause						;	ja, dann Schwellwert holen
		subi	temp, 1							; Eins abziehen
		mov 	count_h, temp					; und nach count_h schreiben
		ldi		count_l, 0xFE					; count_l mit 0xFE besetzen
		rjmp	adconv							; fertig

mk_4:	cbr		status, (1<<START)				; START-Flag war 1: auf 0 setzen
		rjmp 	adconv

notast:	clr		taster

		; Neue Zeiten-Werte ber den AD-Wandler holen

adconv:	inc		adc_int							; ADC-Intervallzhler erhhen
		cpi		adc_int, ADC_INTVL				; Mit Schwelle vergleichen
		brlo	m_done							; 	wenn kleiner, dann fertig
		clr		adc_int							;	wenn grer/gleich, dann zurcksetzen
		rcall	adcread							; ADC-Routine aufrufen

m_done:	rjmp	m_loop							; Endlosschleife




; Timer Overflow-Routine
; **********************

t_ovf:
		;Im Fading-Betrieb: Nchste LED (active_led + 1) anschalten
		
		sbrs	status, FADE					; Fading aktiv ?
		rjmp 	t_done							; 	nein -> weiter zu t_done
		mov		temp, active_led				; 	ja: nchste LED berechnen
		clc										; Carry-Flag vor Shift lschen
		rol		temp							; aktuelle LED weiterschalten
		brcc	not_8	
		ldi		temp, 1
not_8:	out		PORTA, temp

		; Interrupt-Flag setzen (weitere Bearbeitung erfolgt ausserhalb
		;	des Interrupts)

t_done:	sbr		status, (1<<INT_FLAG)			; INT_FLAG setzen
		reti




; Timer Compare-Routine
; *********************

t_comp:
		sbrs 	status, FADE					; Fading aktiv ?
		rjmp	t_ret							; 	nein -> weiter zu t_ret
		out		PORTA, active_led				;	ja: aktuelle LED anschalten
t_ret:	reti





; ADC-Routine
; ***********

adcread:

		; Fading-Poti auslesen und verarbeiten

		sbi		ADMUX, MUX0						; Kanal 9 auswhlen
	
		sbi		ADCSR, ADSC						; Konvertierung starten
adclp1:	sbic	ADCSR, ADSC						; Warten bis Konvertierung beendet
		rjmp	adclp1
		in		temp, ADCH						; Ergebnis holen
		lsr		temp							; Ergebnis durch 8 teilen
		lsr		temp	
		lsr		temp
		mov		fade_int, temp					; Nach fade_int schieben
		ldi		temp, 2							; 2 addieren
		add		fade_int, temp

		; Pausen-Poti auslesen und verarbeiten

		cbi		ADMUX, MUX0						; Kanal 8 auswhlen

		sbi		ADCSR, ADSC						; Konvertierung starten
adclp2:	sbic	ADCSR, ADSC						; Warten bis Konvertierung beendet
		rjmp	adclp2
		in		temp, ADCH						; Ergebnis holen
		cpi 	temp, PAUSE_MIN					; Mit unterer Schwelle vergleichen
		brsh	notmin							;	wenn unter Schwelle,		
		ldi		temp, PAUSE_MIN					; 		Schwellwert nehmen
notmin:	cpi 	temp, (PAUSE_MAX+1)				; Mit oberer Schwelle vergleichen
		brlo	notmax							;	wenn ber Schwelle,
		ldi		temp, PAUSE_MAX					;		Schwellwert nehmen
notmax:	mov		pause, temp						; Pausenschwelle ausgeben

		ret										; fertig

